@@ -1,21 +1,24 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2021 by Dominique Dumont.
+# This software is Copyright (c) 2005-2022 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Exception 2.142;
+package Config::Model::Exception 2.149;
 
 use warnings;
 use strict;
 use Data::Dumper;
 use Mouse;
-use 5.10.1;
+use v5.20;
 use Carp;
 
+use feature qw/postderef signatures/;
+no warnings qw/experimental::postderef experimental::signatures/;
+
 @Carp::CARP_NOT=qw/Config::Model::Exception Config::Model::Exception::Any/;
 
 our $trace = 0;
@@ -45,6 +48,17 @@ has  message => (is => 'rw', isa =>'Str', default => '');
 has  error => (is => 'rw', isa =>'Str', default => '');
 has  trace => (is => 'rw', isa =>'Str', default => '');
 
+# need to keep these objects around: in some tests the error() method is
+# called after the instance is garbage collected. Instances are kept
+# as weak ref in node (and othe tree objects). When instance is
+# garbage collected, it's destroyed so error() can no longer be invoked.
+# Solution: keep instance as error attributes.
+has  instance => ( is => 'rw', isa => 'Ref') ;
+
+sub BUILD ($self, $) {
+    $self->instance($self->object->instance) if defined $self->object;
+}
+
 # without this overload, a test like if ($@) invokes '""' overload
 sub is_error { return ref ($_[0])}
 
@@ -98,19 +112,19 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::Any 2.142;
+package Config::Model::Exception::Any 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception';
 
-package Config::Model::Exception::ModelDeclaration 2.142;
+package Config::Model::Exception::ModelDeclaration 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::Fatal';
 
 sub _desc {'configuration model declaration error' }
 
-package Config::Model::Exception::User 2.142;
+package Config::Model::Exception::User 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::Any';
@@ -118,7 +132,7 @@ sub _desc {'user error' }
 
 
 ## old classes below
-package Config::Model::Exception::Syntax 2.142;
+package Config::Model::Exception::Syntax 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::Any';
@@ -139,7 +153,7 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::LoadData 2.142;
+package Config::Model::Exception::LoadData 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -165,7 +179,7 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::Model 2.142;
+package Config::Model::Exception::Model 2.149;
 
 use Carp;
 use Mouse;
@@ -200,7 +214,7 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::Load 2.142;
+package Config::Model::Exception::Load 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -229,7 +243,7 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::UnavailableElement 2.142;
+package Config::Model::Exception::UnavailableElement 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -260,7 +274,7 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::AncestorClass 2.142;
+package Config::Model::Exception::AncestorClass 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -268,7 +282,7 @@ extends 'Config::Model::Exception::User';
 sub _desc { 'unknown ancestor class'}
 
 
-package Config::Model::Exception::ObsoleteElement 2.142;
+package Config::Model::Exception::ObsoleteElement 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -293,7 +307,7 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::UnknownElement 2.142;
+package Config::Model::Exception::UnknownElement 2.149;
 
 use Carp;
 
@@ -370,14 +384,14 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::WarpError 2.142;
+package Config::Model::Exception::WarpError 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
 
 sub _desc { 'warp error'}
 
-package Config::Model::Exception::Fatal 2.142;
+package Config::Model::Exception::Fatal 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::Any';
@@ -385,7 +399,7 @@ extends 'Config::Model::Exception::Any';
 sub _desc { 'fatal error' }
 
 
-package Config::Model::Exception::UnknownId 2.142;
+package Config::Model::Exception::UnknownId 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -419,7 +433,7 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::WrongValue 2.142;
+package Config::Model::Exception::WrongValue 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -427,7 +441,7 @@ extends 'Config::Model::Exception::User';
 sub _desc { 'wrong value'};
 
 
-package Config::Model::Exception::WrongType 2.142;
+package Config::Model::Exception::WrongType 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
@@ -461,14 +475,14 @@ sub full_message {
     return $msg;
 }
 
-package Config::Model::Exception::ConfigFile 2.142;
+package Config::Model::Exception::ConfigFile 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::User';
 
 sub _desc { 'error in configuration file' }
 
-package Config::Model::Exception::ConfigFile::Missing 2.142;
+package Config::Model::Exception::ConfigFile::Missing 2.149;
 
 use Mouse;
 use Mouse::Util::TypeConstraints;
@@ -487,14 +501,14 @@ sub full_message {
     return "Error: cannot find configuration file " . $self->file . "\n";
 }
 
-package Config::Model::Exception::Formula 2.142;
+package Config::Model::Exception::Formula 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::Model';
 
 sub _desc { 'error in computation formula of the configuration model'}
 
-package Config::Model::Exception::Internal 2.142;
+package Config::Model::Exception::Internal 2.149;
 
 use Mouse;
 extends 'Config::Model::Exception::Fatal';
@@ -517,7 +531,7 @@ Config::Model::Exception - Exception mechanism for configuration model
 
 =head1 VERSION
 
-version 2.142
+version 2.149
 
 =head1 SYNOPSIS
 
@@ -575,7 +589,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2021 by Dominique Dumont.
+This software is Copyright (c) 2005-2022 by Dominique Dumont.
 
 This is free software, licensed under:
 
