@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2021 by Dominique Dumont.
+# This software is Copyright (c) 2005-2022 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::ListId 2.142;
+package Config::Model::ListId 2.149;
 
 use 5.10.1;
 use Mouse;
@@ -50,15 +50,10 @@ sub BUILD {
     }
 
     if ( defined $self->{migrate_keys_from} ) {
-        if ($::_use_log4perl_to_warn) {
-            $user_logger->warn(
-                $self->name, "Using migrate_keys_from with ",
-                "list element is deprecated. Use migrate_values_from");
-        }
-        else {
-            warn $self->name, "Using migrate_keys_from with list element is deprecated.",
-                " Use migrate_values_from\n";
-        }
+        $user_logger->warn(
+            $self->name, "Using migrate_keys_from with ",
+            "list element is deprecated. Use migrate_values_from"
+        );
     }
 
     # Supply the mandatory parameter
@@ -557,7 +552,7 @@ Config::Model::ListId - Handle list element for configuration model
 
 =head1 VERSION
 
-version 2.142
+version 2.149
 
 =head1 SYNOPSIS
 
@@ -752,7 +747,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2021 by Dominique Dumont.
+This software is Copyright (c) 2005-2022 by Dominique Dumont.
 
 This is free software, licensed under:
 
