@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2021 by Dominique Dumont.
+# This software is Copyright (c) 2005-2022 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Role::Grab 2.141;
+package Config::Model::Role::Grab 2.149;
 
 # ABSTRACT: Role to grab data from elsewhere in the tree
 
@@ -15,11 +15,17 @@ use Mouse::Role;
 use strict;
 use warnings;
 use Carp;
+use 5.20.0;
 
 use List::MoreUtils qw/any/;
 use Mouse::Util;
 use Log::Log4perl qw(get_logger :levels);
 
+with "Config::Model::Role::Utils";
+use feature qw/signatures postderef/;
+no warnings qw/experimental::signatures experimental::postderef/;
+
+
 my $logger = get_logger("Grab");
 
 ## Navigation
@@ -31,14 +37,12 @@ my $logger = get_logger("Grab");
 
 # Now return an object and not a value !
 
-sub grab {
-    my $self = shift;
+sub grab ($self, @args) {
+    my %args = _resolve_arg_shortcut(\@args, 'steps');
     my ( $steps, $mode, $autoadd, $type, $grab_non_available, $check ) =
         ( undef, 'strict', 1, undef, 0, 'yes' );
 
-    my %args = @_ > 1 ? @_ : ( steps => $_[0] );
-
-    $steps               = delete $args{steps} // delete $args{step};
+    $steps              = delete $args{steps} // delete $args{step};
     $mode               = delete $args{mode} if defined $args{mode};
     $autoadd            = delete $args{autoadd} if defined $args{autoadd};
     $grab_non_available = delete $args{grab_non_available}
@@ -61,6 +65,8 @@ sub grab {
 
     # accept commands, grep remove empty items left by spurious spaces
     my $huge_string = ref $steps ? join( ' ', @$steps ) : $steps;
+    return $self unless $huge_string;
+
     my @command = (
         $huge_string =~ m/
          (         # begin of *one* command
@@ -159,7 +165,7 @@ COMMAND:
         }
 
         {
-            no warnings "uninitialized";
+            no warnings "uninitialized"; ## no critic (TestingAndDebugging::ProhibitNoWarnings)
             $logger->debug("grab: cmd '$cmd' -> name '$name', action '$action', arg '$arg'");
         }
 
@@ -266,9 +272,8 @@ COMMAND:
     return wantarray ? ( $return, @command ) : $return;
 }
 
-sub grab_value {
-    my $self = shift;
-    my %args = scalar @_ == 1 ? ( steps => $_[0] ) : @_;
+sub grab_value ($self, @args) {
+    my %args = _resolve_arg_shortcut(\@args, 'steps');
 
     my $obj = $self->grab(%args);
 
@@ -277,16 +282,15 @@ sub grab_value {
 
     return if ( $args{mode} and $args{mode} eq 'loose' and not defined $obj );
 
-    Config::Model::Exception::User->throw(
-        object  => $self,
-        message => "grab_value: cannot get value of non-leaf or check_list "
-            . "item with '"
-            . join( "' '", @_ )
-            . "'. item is $obj"
-        )
-        unless ref $obj
-        and ( $obj->isa("Config::Model::Value")
-        or $obj->isa("Config::Model::CheckList") );
+    if (not $obj->isa("Config::Model::Value")
+        and not $obj->isa("Config::Model::CheckList")
+    ) {
+        Config::Model::Exception::User->throw(
+            object  => $self,
+            message => "Cannot get a value from '". $obj->location . "'. ",
+            info    => "grab arguments are '".join( "' '", @args ) . "'."
+        );
+    }
 
     my $value = $obj->fetch;
     if ( $logger->is_debug ) {
@@ -296,13 +300,8 @@ sub grab_value {
     return $value;
 }
 
-sub grab_annotation {
-    my $self = shift;
-    my @args = scalar @_ == 1 ? ( steps => $_[0] ) : @_;
-
-    my $obj = $self->grab(@args);
-
-    return $obj->annotation;
+sub grab_annotation ($self, @args) {
+    return $self->grab(@args)->annotation;
 }
 
 sub grab_root {
@@ -354,6 +353,7 @@ sub grab_ancestor_with_element_named {
             );
         }
     }
+    return; # should never be reached...
 }
 
 1;
@@ -370,7 +370,7 @@ Config::Model::Role::Grab - Role to grab data from elsewhere in the tree
 
 =head1 VERSION
 
-version 2.141
+version 2.149
 
 =head1 SYNOPSIS
 
@@ -504,7 +504,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2021 by Dominique Dumont.
+This software is Copyright (c) 2005-2022 by Dominique Dumont.
 
 This is free software, licensed under:
 
